﻿using System;
using System.Configuration;
using System.Windows;
using System.Windows.Threading;
using System.Collections.Generic;

using Microsoft.Lync.Model;
using Microsoft.Lync.Model.Extensibility;
using Microsoft.Lync.Model.Conversation;
using Microsoft.Lync.Model.Conversation.Sharing;

namespace LyncApplicationSharing
{
    public partial class MainWindow : Window
    {
        // Conversation created by the application
        private Conversation _conversation;

        // Modality of the local user
        private ApplicationSharingModality _localModality;

        // Modality of the second user in the conversation
        private ApplicationSharingModality _remoteModality;

        public MainWindow()
        {
            InitializeComponent();
            listResources.DisplayMemberPath = "Name";
        }

        private void StartConversation()
        {
            var automaton = LyncClient.GetAutomation();
            string[] recipients = { ConfigurationManager.AppSettings["SecondaryLabUserId"] };

            // Send a message to start the conversation
            Dictionary<AutomationModalitySettings, object> settings = new Dictionary<AutomationModalitySettings, object>();
            settings.Add(AutomationModalitySettings.FirstInstantMessage, "Lync Wave 15 lab starting.");
            settings.Add(AutomationModalitySettings.SendFirstInstantMessageImmediately, true);
         
            var call = automaton.BeginStartConversation(
                AutomationModalities.InstantMessage, 
                recipients, 
                settings, 
                (ar) =>
                {
                    try
                    {
                        if (_conversation == null)
                        {
                            _conversation = automaton.EndStartConversation(ar).Conversation;
                            _conversation.StateChanged += OnConversationState_Changed;

                            //TODO: 2.1.1 - Acquire sharing modality for the conversation
                            

                            foreach (Participant p in _conversation.Participants)
                            {
                                if (!p.IsSelf)
                                {
                                    //TODO: 2.1.2 - Acquire sharing modality from the remote user
                                    
                                    break;
                                }
                            }

                            //TODO: 2.1.3 - Attach event handlers to the conversation's sharing modality
                            

                            //TODO: 2.1.4 - Attach event handlers to the remote user's sharing modality.
                            

                            RefreshResources();

                            Dispatcher.BeginInvoke(DispatcherPriority.Normal,
                            (Action)delegate()
                            {
                                btnConvoStart.IsEnabled = false;
                                btnConvoEnd.IsEnabled = true;
                                btnShareStart.IsEnabled = true;
                            });
                        }
                    }
                    catch (Exception ex)
                    {
                        Log(string.Format("Error starting conversation: {0}", ex.Message));
                    }
                }, null);
        }

        private void EndConversation()
        {
            if (_conversation != null)
            {
                _conversation.End();
            }
        }

        private void OnConversationState_Changed(object sender, ConversationStateChangedEventArgs e)
        {
            Log(string.Format("Conversation state changed to {0}.", e.NewState.ToString()));

            if (e.NewState == ConversationState.Terminated)
            {
                _localModality = null;
                _conversation = null;

                Dispatcher.BeginInvoke(DispatcherPriority.Normal,
                (Action)delegate()
                {
                    listEvents.Items.Clear();
                    listResources.Items.Clear();

                    btnConvoStart.IsEnabled = true;
                    btnShareStart.IsEnabled = btnShareEnd.IsEnabled = btnConvoEnd.IsEnabled = btnAcceptRequest.IsEnabled = 
                        btnDeclineRequest.IsEnabled = btnGrantControl.IsEnabled = btnRevokeControl.IsEnabled = false;
                });
            }
        }

        // Called whenever the local resources list gets changed, such as when a new shareable process starts up
        void OnLocalSharedResources_Changed(object sender, LocalSharedResourcesChangedEventArgs e)
        {
            Log("Local resources changed event called.");
            RefreshResources();
        }

        private void OnParticipationState_Changed(object sender, ParticipationStateChangedEventArgs e)
        {
            Log(string.Format("Participation state changed to {0} for {1}.", e.NewState.ToString(), 
                (string)((ApplicationSharingModality)sender).Participant.Properties[ParticipantProperty.Name]));
        }

        // Called whenever the controller of the screen share is changed (such as if user A lets user B control his screen)
        private void OnController_Changed(object sender, ControllerChangedEventArgs e)
        {
            string name = e.ControllerName;

            // An empty controller name means that the controller is now the local user
            if (string.IsNullOrEmpty(e.ControllerName))
            {
                name = LyncClient.GetClient().Self.Contact.Uri;
            }

            Log(string.Format("Controller changed to {0}.", name));
        }

        // Called whenever a user other than the owner of the screen share seeks to take control of the screen share
        private void OnControlRequest_Received(object sender, ControlRequestReceivedEventArgs e)
        {
            Log(string.Format("Control request received from {0}", e.Requester.Properties[ParticipantProperty.Name].ToString()));
        }

        // Called whenever the modality state is changed
        private void OnModalityState_Changed(object sender, ModalityStateChangedEventArgs e)
        {
            Log(string.Format("Modality state changed to {0}", e.NewState.ToString()));
        }

        // Called whenever the ability of a user to perform a certain action on the screen share session is changed
        //  An example would be when a user takes control of a screen share session, the original owner's ability to 
        //  revoke the new holder's ownership of the screen is activated
        private void OnActionAvailability_Changed(object sender, ModalityActionAvailabilityChangedEventArgs e)
        {
            Log(string.Format("Action {0} is {1} for {2}.", 
                e.Action.ToString(), e.IsAvailable ? "available" : "unavailable",
                (string)((ApplicationSharingModality)sender).Participant.Properties[ParticipantProperty.Name]));

            System.Windows.Controls.Button btnToChange = null;

            switch (e.Action)
            {
                // These two modalities represent a user responding to a request to screenshare.
                case ModalityAction.AcceptSharingControlRequest:
                    btnToChange = btnAcceptRequest;
                    break;
                case ModalityAction.DeclineSharingControlRequest:
                    btnToChange = btnDeclineRequest;
                    break;
                // These two modalities represent sharing and stopping sharing of a screen.
                case ModalityAction.Connect:
                    btnToChange = btnShareStart;
                    break;
                case ModalityAction.Disconnect:
                    btnToChange = btnShareEnd;
                    break;
                // These two modalities represent the user granting a user the right to share (without being asked) and taking back control
                case ModalityAction.GrantSharingControl:
                    btnToChange = btnGrantControl;
                    break;
                case ModalityAction.RevokeSharingControl:
                    btnToChange = btnRevokeControl;
                    break;
            }

            if (btnToChange != null)
            {
                Dispatcher.BeginInvoke(DispatcherPriority.Normal,
                    (Action)delegate()
                    {
                        btnToChange.IsEnabled = e.IsAvailable;
                    });
            }
        }

        private void StartConversation_Click(object sender, RoutedEventArgs e)
        {
            StartConversation();
        }

        private void EndConversation_Click(object sender, RoutedEventArgs e)
        {
            EndConversation();
        }

        private void ShareResource_Click(object sender, RoutedEventArgs e)
        {
            ShareResource();
        }

        private void EndShareResource_Click(object sender, RoutedEventArgs e)
        {
            EndShareResource();
        }

        private void AcceptControl_Click(object sender, RoutedEventArgs e)
        {
            AcceptControlRequest();
        }

        private void DeclineControl_Click(object sender, RoutedEventArgs e)
        {
            DeclineControlRequest();
        }

        private void GrantControl_Click(object sender, RoutedEventArgs e)
        {
            GrantControl();
        }

        private void RevokeControl_Click(object sender, RoutedEventArgs e)
        {
            RevokeControl();
        }

        private void RefreshResources()
        {
            Dispatcher.BeginInvoke(DispatcherPriority.Normal,
                (Action)delegate()
                {
                    listResources.Items.Clear();

                    //TODO: 2.1.5 - Acquire the list of local shareable resources
                    

                    listResources.SelectedIndex = 0;
                });
        }

        void ShareResource()
        {
            try
            {
                var resource = (SharingResource)listResources.SelectedItem;

                //TODO: 2.1.6 - Share a single resource in the conversation
                
            }
            catch (Exception ex)
            {
                Log(string.Format("Error at BeginShareResources: {0}", ex.Message));
            }
        }

        void EndShareResource()
        {
            try
            {
                //TODO: 2.1.7 - Disconnect a resource from the conversation
                
            }
            catch (Exception ex)
            {
                Log(string.Format("Error at BeginDisconnect: {0}", ex.Message));
            }

        }

        void AcceptControlRequest()
        {
            if (_remoteModality != null)
            {
                try
                {
                    //TODO: 2.1.8 - Accept a control request from the remote user
                                
                }
                 catch (Exception ex)
                 {
                     Log(string.Format("Error at BeginAcceptControlRequest: {0}", ex.Message));
                 }
            }
        }

        void DeclineControlRequest()
        {
            if (_remoteModality != null)
            {
                try
                {
                    //TODO: 2.1.9 - Decline a control request from the remote user
                    
                }
                catch (Exception ex)
                {
                    Log(string.Format("Error at BeginDeclineControlRequest: {0}", ex.Message));
                }
            }
        }

        void GrantControl()
        {
            try
            {
                //TODO: 2.1.10 - Grant control to the remote user
                
            }
            catch (Exception ex)
            {
                Log(string.Format("Error at BeginGrantControl: {0}", ex.Message));
            }
        }

        void RevokeControl()
        {
            try
            {
                //TODO: 2.1.11 - Revoke sharing control
                
            }
            catch (Exception ex)
            {
                Log(string.Format("Error at BeginRevokeControl: {0}", ex.Message));
            }
        }

        private void Log(string message)
        {
            Dispatcher.BeginInvoke(
                DispatcherPriority.Normal,
                (Action)delegate()
                {
                    listEvents.Items.Insert(0, string.Format("{0} {1}", DateTime.Now.ToString(), message));
                });
        }
    }
}
